using LabApi.Events.Arguments.PlayerEvents;
using LabApi.Features.Wrappers;
using MapGeneration;
using MEC;
using Mirror;
using PlayerRoles;
using Respawning;
using UnityEngine;
using Random = UnityEngine.Random;

namespace GamblingCoin;

public class GamblingCoinEventHandler
{
    private readonly WeightedRandomExecutor<PlayerFlippedCoinEventArgs> _executor;

    public GamblingCoinEventHandler()
    {
        var configMessages = Plugin.Singleton.ConfigMessages;
        var configGameplay = Plugin.Singleton.ConfigGameplay;
        var configChances = Plugin.Singleton.ConfigChances;

        _executor = new WeightedRandomExecutor<PlayerFlippedCoinEventArgs>();

        _executor
            .AddAction(x =>
            {
                Warhead.DetonationTime += configGameplay.WarheadTimeIncrease;
                Warhead.Start(suppressSubtitles: true);
                
                foreach (var player in Player.ReadyList)
                {
                    player.SendBroadcast($"{x.Player.Nickname} activated the Alpha Warhead through gambling! (coin)", 10);
                }
            }, configChances.NukeChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.SpawnWaveMessage, configGameplay.BroadcastDuration);

                if (GetPlayers().Any(player => player.Role == RoleTypeId.Spectator))
                    WaveManager.InitiateRespawn(WaveManager.Waves[Random.Range(0, WaveManager.Waves.Count)]);
            }, configChances.SpawnWaveChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.ItemSpawnMessage, configGameplay.BroadcastDuration);
                SpawnRandomItemAtPlayer(x.Player, configGameplay.Items.CommonItems);
            }, configChances.CommonItemChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.UncommonItemSpawnMessage, configGameplay.BroadcastDuration);
                SpawnRandomItemAtPlayer(x.Player, configGameplay.Items.UncommonItems);
            }, configChances.UncommonItemChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.RareItemSpawnMessage, configGameplay.BroadcastDuration);
                SpawnRandomItemAtPlayer(x.Player, configGameplay.Items.RareItems);
            }, configChances.RareItemChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.EpicItemSpawnMessage, configGameplay.BroadcastDuration);
                SpawnRandomItemAtPlayer(x.Player, configGameplay.Items.EpicItems);
            }, configChances.EpicItemChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.LegendaryItemSpawnMessage, configGameplay.BroadcastDuration);

                var player = x.Player;
                var items = configGameplay.Items.LegendaryItems;

                var itemIndex = Random.Range(0, items.Length);
                var item = items[itemIndex];
                
                var pickup = Pickup.Create(item, player.Position + new Vector3(0, 1, 0));
                if (pickup == null) return;

                pickup.Spawn();
            }, configChances.LegendaryItemChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.RandomTeleportMessage, configGameplay.BroadcastDuration);
                var randomRoom = Map.GetRandomRoom();
                if (randomRoom == null) return;

                var isInOtherZone = randomRoom is { Zone: FacilityZone.Other };
                var isDetonated = Warhead.IsDetonated;
                var isDecontaminating = Decontamination.IsDecontaminating;

                var isInDetonatedZone = isDetonated && randomRoom.Zone is FacilityZone.HeavyContainment
                    or FacilityZone.LightContainment or FacilityZone.Entrance;
                var isInDecontaminatedZone = isDecontaminating && randomRoom.Zone is FacilityZone.LightContainment;
                while (isInOtherZone || isInDetonatedZone || isInDecontaminatedZone)
                {
                    randomRoom = Map.GetRandomRoom();
                    if (randomRoom == null) return;

                    isInOtherZone = randomRoom is { Zone: FacilityZone.Other };
                    isInDetonatedZone = isDetonated && randomRoom.Zone is FacilityZone.HeavyContainment
                        or FacilityZone.LightContainment or FacilityZone.Entrance;
                    isInDecontaminatedZone = isDecontaminating && randomRoom.Zone is FacilityZone.LightContainment;
                }

                var newPos = randomRoom.Position;

                x.Player.Position = newPos + new Vector3(0, configGameplay.TeleportHeightOffset, 0);
            }, configChances.RandomTeleportChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.StealItemMessage, configGameplay.BroadcastDuration);

                if (x.Player.CurrentItem != null) x.Player.DropItem(x.Player.CurrentItem);
            }, configChances.StealItemChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.ExplosionMessage, configGameplay.BroadcastDuration);

                x.Player.ClearInventory();

                TimedGrenadeProjectile.SpawnActive(x.Player.Position, ItemType.GrenadeHE, x.Player);
            }, configChances.ExplosionChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.AntiMicroMessage, configGameplay.BroadcastDuration);
                GetPlayers().ForEach(p => { p.RemoveItem(ItemType.MicroHID, configGameplay.MaxMicrosToRemove); });
                
                foreach (var microHidPickup in MicroHIDPickup.List)
                {
                    microHidPickup.Destroy();
                }
            }, configChances.AntiMicroChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.GrenadeMessage, configGameplay.BroadcastDuration);

                TimedGrenadeProjectile.SpawnActive(x.Player.Position, ItemType.GrenadeHE, x.Player, 2);
            }, configChances.GrenadeChance)
            .AddAction(x =>
                {
                    x.Player.SendBroadcast(configMessages.PocketDimensionMessage, configGameplay.BroadcastDuration);

                    PocketDimension.ForceInside(x.Player);
                },
                configChances.PocketDimensionChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.AdvancedPositiveEffectMessage, configGameplay.BroadcastDuration);
                ApplyRandomEffect(x.Player, configGameplay.Effects.AdvancedPositive);
            }, configChances.AdvancedPositiveEffectChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.PositiveEffectMessage, configGameplay.BroadcastDuration);
                ApplyRandomEffect(x.Player, configGameplay.Effects.Positive);
            }, configChances.PositiveEffectChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.NegativeEffectMessage, configGameplay.BroadcastDuration);
                ApplyRandomEffect(x.Player, configGameplay.Effects.Negative);
            }, configChances.NegativeEffectChance)
            .AddAction(x =>
            {
                x.Player.SendBroadcast(configMessages.AdvancedNegativeEffectMessage, configGameplay.BroadcastDuration);
                ApplyRandomEffect(x.Player, configGameplay.Effects.AdvancedNegative);
            }, configChances.AdvancedNegativeEffectChance)
            .AddAction(x =>
            {
                var players = GetPlayers().Where(player=>player.Team is not Team.Dead and not Team.SCPs).ToArray();

                var randomPlayer = players[Random.Range(0, GetPlayers().Length)];

                x.Player.SendBroadcast(configMessages.SwitchInventoryMessage, configGameplay.BroadcastDuration);
                randomPlayer.SendBroadcast(configMessages.SwitchInventoryMessage, configGameplay.BroadcastDuration);

                var randomPlayerItems = new List<Item>();
                randomPlayer.Items.CopyTo(randomPlayerItems);

                List<KeyValuePair<ItemType, ushort>> randomPlayerAmmo = new();
                randomPlayer.Ammo.CopyTo(randomPlayerAmmo);
                var items = x.Player.Items;
                var ammoCount = x.Player.Ammo;

                randomPlayer.ClearInventory();
                foreach (var itemBase in items) randomPlayer.AddItem(itemBase.Type);
                foreach (var ammo in ammoCount) randomPlayer.AddAmmo(ammo.Key, ammo.Value);

                x.Player.ClearInventory();
                foreach (var randomPlayerItem in randomPlayerItems) x.Player.AddItem(randomPlayerItem.Type);
                foreach (var randomPlayerAmmoItem in randomPlayerAmmo) x.Player.AddAmmo(randomPlayerAmmoItem.Key, randomPlayerAmmoItem.Value);
            }, configChances.SwitchInventoryChance)
            .AddAction(x => { x.Player.CurrentItem?.DropItem().Destroy(); }, configChances.RemoveCoinChance)
            .AddAction(x =>
            {
                var spectators = Player.List.Where(player => player.Role == RoleTypeId.Spectator).ToArray();
                if(spectators.Length == 0) return;
                
                var spectator = spectators[Random.Range(0, spectators.Length-1)];
                
                x.Player.SendBroadcast(configMessages.SpawnZombieMessage, configGameplay.BroadcastDuration);
                spectator.SetRole(RoleTypeId.Scp0492);

                Timing.CallDelayed(0.5f, () =>
                {
                    var spawnRoom = Map.Rooms.First(room => room.Name == RoomName.HczWarhead);
                    if (Warhead.IsDetonated)
                    {
                        spawnRoom = Map.Rooms.First(room => room.Name == RoomName.Outside);
                    }

                    spectator.Position = spawnRoom.Position + new Vector3(0, 1, 0);
                });
            }, configChances.SpawnZombieChance);

        return;

        void ApplyRandomEffect(Player player, AdvancedEffectSettings settings)
        {
            var effectChosen = settings.Effects[Random.Range(0, settings.Effects.Length)];
            var effectSettings = settings.Settings[effectChosen];

            player.ReferenceHub.playerEffectsController.ChangeState(effectChosen, effectSettings.Intensity,
                effectSettings.Duration, effectSettings.AddDuration);
        }

        void SpawnItemAtPlayer(Player player, ItemType item)
        {
            var pickup = Pickup.Create(item, player.Position + new Vector3(0, 1, 0));
            if (pickup == null) return;

            pickup.Spawn();
        }

        void SpawnRandomItemAtPlayer(Player player, ItemType[] items)
        {
            var itemIndex = Random.Range(0, items.Length);
            SpawnItemAtPlayer(player, items[itemIndex]);
        }

        Player[] GetPlayers()
        {
            return Player.ReadyList.ToArray();
        }
    }

    public void OnFlippedCoin(PlayerFlippedCoinEventArgs ev)
    {
        _executor.Execute(ev);
    }
}